@@ -146,11 +146,16 @@ public class FilteringParserDelegate extends JsonParserDelegate
      */
 
     @Override public JsonToken getCurrentToken() { return _currToken; }
+    @Override public JsonToken currentToken() { return _currToken; }
 
     @Override public final int getCurrentTokenId() {
         final JsonToken t = _currToken;
         return (t == null) ? JsonTokenId.ID_NO_TOKEN : t.id();
     }
+    @Override public final int currentTokenId() {
+        final JsonToken t = _currToken;
+        return (t == null) ? JsonTokenId.ID_NO_TOKEN : t.id();
+    }
 
     @Override public boolean hasCurrentToken() { return _currToken != null; }
     @Override public boolean hasTokenId(int id) {
@@ -221,6 +226,21 @@ public class FilteringParserDelegate extends JsonParserDelegate
     @Override
     public JsonToken nextToken() throws IOException
     {
+    	//Check for _allowMultipleMatches - false and atleast there is one token - which is _currToken
+    	// check for no buffered context _exposedContext - null
+    	//If all the conditions matches then check for scalar / non-scalar property
+    	if(!_allowMultipleMatches && _currToken != null && _exposedContext == null){
+    		//if not scalar and ended successfully, then return null
+    		if((_currToken.isStructEnd()  && _headContext.isStartHandled()) ){
+    			return (_currToken = null);
+    		}
+    		//else if scalar, and scalar not present in obj/array and !includePath and INCLUDE_ALL matched once
+    		// then return null 
+    		else if(_currToken.isScalarValue() && !_headContext.isStartHandled() && !_includePath 
+    				&& _itemFilter == TokenFilter.INCLUDE_ALL) {
+    			return (_currToken = null);
+    		}
+    	}
         // Anything buffered?
         TokenFilterContext ctxt = _exposedContext;
 
@@ -266,7 +286,8 @@ public class FilteringParserDelegate extends JsonParserDelegate
         JsonToken t = delegate.nextToken();
         if (t == null) {
             // no strict need to close, since we have no state here
-            return (_currToken = t);
+            _currToken = t;
+            return t;
         }
 
         // otherwise... to include or not?
@@ -438,7 +459,8 @@ public class FilteringParserDelegate extends JsonParserDelegate
         while (true) {
             JsonToken t = delegate.nextToken();
             if (t == null) { // is this even legal?
-                return (_currToken = t);
+                _currToken = t;
+                return t;
             }
             TokenFilter f;
 
@@ -877,4 +899,5 @@ public class FilteringParserDelegate extends JsonParserDelegate
         }
         return _headContext;
     }
+  
 }
